<?php declare(strict_types=1);

namespace Convo\Proto;


use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Psr\Http\Message\ResponseInterface;

/**
 * @author Tole
 * 
 * Simple middleware for enabling cross-domain api access for client side JavaScript applications.
 *
 */
class CorsMiddleware implements \Psr\Http\Server\MiddlewareInterface
{	
	
	/**
	 * @var \Psr\Log\LoggerInterface
	 */
	private $_logger;
	
	/**
	 * @var \Convo\Core\Util\IHttpFactory
	 */
	private $_httpFactory;
	
	/**
	 * @var string
	 */
	private $_domain;
	
	public function __construct( \Psr\Log\LoggerInterface $logger, $httpFactory, $domain)
	{
		$this->_logger	=	$logger;
		$this->_httpFactory = $httpFactory;
		$this->_domain = $domain;
	}
	
	public function process( ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
	{
		if ( $request->getMethod() === 'OPTIONS') {
		
			$this->_logger->debug( 'Dumping all headers to OPTIONS response');
			
			$response	=	$this->_httpFactory->buildResponse( null);
			$response	=	$response->withHeader( 'Access-Control-Allow-Headers', 'Origin, Content-Type, Accept, Authorization, X-Request-With, Set-Cookie, Cookie, Bearer');
			$response	=	$response->withHeader( 'Access-Control-Allow-Methods', 'GET,POST,OPTIONS,DELETE,PUT');
			$response	=	$response->withHeader( 'Access-Control-Max-Age', 60 * 60 * 24 * 7);
			$response	=	$response->withHeader( 'Access-Control-Allow-Origin', $this->_domain);
			$response	=	$response->withHeader( 'Access-Control-Allow-Credentials', 'true');
			
			return $response;
		}
		
		$this->_logger->debug( 'Propagating to request handler ['.get_class( $handler).']');
		
		$response	=	$handler->handle( $request);
		$response	=	$response->withHeader( 'Access-Control-Allow-Origin', $this->_domain);
		$response	=	$response->withHeader( 'Access-Control-Allow-Credentials', 'true');
		
		return $response;
	}
	
	// UTIL
	public function __toString()
	{
		return get_class( $this).'[]';
	}
}