const path = require('path');

var pluginProvide = require('./webpack/plugin-provide');
var pluginCopy = require('./webpack/plugin-copy');
var pluginHtml = require('./webpack/plugin-html');
var pluginUglify = require('./webpack/plugin-uglify');
var loadEnv = require('./webpack/load-env');
var optimization = require('./webpack/optimization');
var externals = require('./webpack/externals');
var rules = require('./webpack/module-rules');

module.exports = function (env) {
    const config = loadEnv(env.ENV);

    let configFilename = '';

    if (!['dev', 'local'].includes(env.ENV)) {
        configFilename = `__config.${env.ENV}.php`;
    } else {
        configFilename = '__config.php';
    }

    let webpackConfig = {
        mode: config['ENV_MODE'],
        entry: {
            main: path.resolve('app/proto', 'app.js'),
        },
        output: {
            path: path.resolve(__dirname, 'dist/www'),
            publicPath: '/',
            filename: '[name].js',
            chunkFilename: '[name].js',
        },
        externals: externals,
        optimization: optimization,
        module: { rules },
        devServer: {
            historyApiFallback: true,
            host: '127.0.0.1',
            hot: true
        },
        resolve: {
            alias: {
                'jquery-ui': 'jquery-ui-dist/jquery-ui.js'
            }
        }
    };

    webpackConfig['plugins'] = [
        pluginCopy(configFilename),
        pluginProvide(),
        pluginHtml(config, './app/proto/index.ejs', 'index.html')
    ];

    if (!['dev', 'local'].includes(env.ENV)) {
        webpackConfig['plugins'].push(pluginUglify());
    }

    if (config['DEVTOOL']) {
        webpackConfig['devtool'] = config['DEVTOOL'];
    }

    return webpackConfig;
};
